@@ -26,7 +26,7 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <libindicator/indicator-object.h>
 #include "libappindicator/app-indicator.h"
-#include "app-indicator-enum-types.h"
+#include "libappindicator/app-indicator-enum-types.h"
 #include "application-service-appstore.h"
 #include "application-service-marshal.h"
 #include "dbus-shared.h"
@@ -142,6 +142,7 @@ static void approver_free (gpointer papprover, gpointer user_data);
 static void check_with_new_approver (gpointer papp, gpointer papprove);
 static void check_with_old_approver (gpointer papprove, gpointer papp);
 static Application * find_application (ApplicationServiceAppstore * appstore, const gchar * address, const gchar * object);
+static Application * find_application_by_menu (ApplicationServiceAppstore * appstore, const gchar * address, const gchar * menuobject);
 static void bus_get_cb (GObject * object, GAsyncResult * res, gpointer user_data);
 static void dbus_proxy_cb (GObject * object, GAsyncResult * res, gpointer user_data);
 static void app_receive_signal (GDBusProxy * proxy, gchar * sender_name, gchar * signal_name, GVariant * parameters, gpointer user_data);
@@ -263,31 +264,20 @@ bus_method_call (GDBusConnection * connection, const gchar * sender,
 {
 	ApplicationServiceAppstore * service = APPLICATION_SERVICE_APPSTORE(user_data);
 	GVariant * retval = NULL;
+	Application *app = NULL;
+	const gchar *dbusaddress;
+	const gchar *dbusmenuobject;
 
 	if (g_strcmp0(method, "GetApplications") == 0) {
 		retval = get_applications(service);
 	} else if (g_strcmp0(method, "ApplicationScrollEvent") == 0) {
-		Application *app = NULL;
-		const gchar *dbusaddress;
-		const gchar *dbusobject;
 		gchar *orientation = NULL;
 		gint delta;
 		guint direction;
 
-		g_variant_get (params, "(&s&siu)", &dbusaddress, &dbusobject,
+		g_variant_get (params, "(&s&siu)", &dbusaddress, &dbusmenuobject,
 		                                   &delta, &direction);
 
-		GList *l;
-		for (l = service->priv->applications; l != NULL; l = l->next) {
-			Application *a = l->data;
-
-			if (g_strcmp0(a->dbus_name, dbusaddress) == 0 &&
-			      g_strcmp0(a->menu, dbusobject) == 0) {
-			   app = a;
-			   break;
-			}
-		}
-
 		switch (direction) {
 			case INDICATOR_OBJECT_SCROLL_UP:
 				delta = -delta;
@@ -301,9 +291,22 @@ bus_method_call (GDBusConnection * connection, const gchar * sender,
 				orientation = "horizontal";
 		}
 
+		app = find_application_by_menu(service, dbusaddress, dbusmenuobject);
+
 		if (app != NULL && app->dbus_proxy != NULL && orientation != NULL) {
 			g_dbus_proxy_call(app->dbus_proxy, "Scroll",
-			              	  g_variant_new("(is)", delta, orientation),
+			                  g_variant_new("(is)", delta, orientation),
+			                  G_DBUS_CALL_FLAGS_NONE, -1, NULL, NULL, NULL);
+		}
+	} else if (g_strcmp0(method, "ApplicationSecondaryActivateEvent") == 0) {
+		guint time;
+
+		g_variant_get (params, "(&s&su)", &dbusaddress, &dbusmenuobject, &time);
+		app = find_application_by_menu(service, dbusaddress, dbusmenuobject);
+
+		if (app != NULL && app->dbus_proxy != NULL) {
+			g_dbus_proxy_call(app->dbus_proxy, "XAyatanaSecondaryActivate",
+			                  g_variant_new("(u)", time),
 			                  G_DBUS_CALL_FLAGS_NONE, -1, NULL, NULL, NULL);
 		}
 	} else {
@@ -505,6 +508,14 @@ got_all_properties (GObject * source_object, GAsyncResult * res,
 	else {
 		app->validated = TRUE;
 
+		/* It is possible we're coming through a second time and
+		   getting the properties.  So we need to ensure we don't
+		   already have them stored */
+		g_free(app->id);
+		g_free(app->category);
+		g_free(app->icon);
+		g_free(app->menu);
+
 		app->id = g_variant_dup_string(id, NULL);
 		app->category = g_variant_dup_string(category, NULL);
 		app->status = string_to_status(g_variant_get_string(status, NULL));
@@ -513,10 +524,30 @@ got_all_properties (GObject * source_object, GAsyncResult * res,
 
 		/* Now the optional properties */
 
-		if (aicon_name != NULL) {
-			app->aicon = g_variant_dup_string(aicon_name, NULL);
+		g_free(app->icon_desc);
+		if (icon_desc != NULL) {
+			app->icon_desc = g_variant_dup_string(icon_desc, NULL);
+		}
+		else {
+			app->icon_desc = g_strdup("");
 		}
 
+		g_free(app->aicon);
+		if (aicon_name != NULL) {
+			app->aicon = g_variant_dup_string(aicon_name, NULL);
+		} else {
+			app->aicon = g_strdup("");
+		}
+
+		g_free(app->aicon_desc);
+		if (aicon_desc != NULL) {
+			app->aicon_desc = g_variant_dup_string(aicon_desc, NULL);
+		}
+		else {
+			app->aicon_desc = g_strdup("");
+		}
+
+		g_free(app->icon_theme_path);
 		if (icon_theme_path != NULL) {
 			app->icon_theme_path = g_variant_dup_string(icon_theme_path, NULL);
 		} else {
@@ -536,12 +567,14 @@ got_all_properties (GObject * source_object, GAsyncResult * res,
 		g_debug("'%s' ordering index is '%X'", app->id, app->ordering_index);
 		app->appstore->priv->applications = g_list_sort_with_data(app->appstore->priv->applications, app_sort_func, NULL);
 
+		g_free(app->label);
 		if (label != NULL) {
 			app->label = g_variant_dup_string(label, NULL);
 		} else {
 			app->label = g_strdup("");
 		}
 
+		g_free(app->guide);
 		if (guide != NULL) {
 			app->guide = g_variant_dup_string(guide, NULL);
 		} else {
@@ -733,6 +766,9 @@ application_free (Application * app)
 	if (app->icon_desc != NULL) {
 		g_free(app->icon_desc);
 	}
+	if (app->aicon != NULL) {
+		g_free(app->aicon);
+	}
 	if (app->aicon_desc != NULL) {
 		g_free(app->aicon_desc);
 	}
@@ -870,12 +906,12 @@ apply_status (Application * app)
 		if (app->visible_state == VISIBLE_STATE_HIDDEN) {
 			/* Put on panel */
 			emit_signal (appstore, "ApplicationAdded",
-				     g_variant_new ("(sisossss)", newicon,
+				     g_variant_new ("(sisosssss)", newicon,
 			                            get_position(app),
 			                            app->dbus_name, app->menu,
 			                            app->icon_theme_path,
 			                            app->label, app->guide,
-			                            newdesc));
+			                            newdesc, app->id));
 		} else {
 			/* Icon update */
 			gint position = get_position(app);
@@ -1199,6 +1235,28 @@ find_application (ApplicationServiceAppstore * appstore, const gchar * address,
 	return NULL;
 }
 
+/* Looks for an application in the list of applications with the matching menu */
+static Application *
+find_application_by_menu (ApplicationServiceAppstore * appstore, const gchar * address, const gchar * menuobject)
+{
+	g_return_val_if_fail(appstore, NULL);
+	g_return_val_if_fail(address, NULL);
+	g_return_val_if_fail(menuobject, NULL);
+
+	ApplicationServiceAppstorePrivate * priv = appstore->priv;
+	GList *l;
+
+	for (l = priv->applications; l != NULL; l = l->next) {
+		Application *a = l->data;
+		if (g_strcmp0(a->dbus_name, address) == 0 &&
+		      g_strcmp0(a->menu, menuobject) == 0) {
+			return a;
+		}
+	}
+
+	return NULL;
+}
+
 /* Removes an application.  Currently only works for the apps
    that are shown. */
 void
@@ -1265,19 +1323,20 @@ get_applications (ApplicationServiceAppstore * appstore)
 				continue;
 			}
 
-			g_variant_builder_add (&builder, "(sisossss)", app->icon,
+			g_variant_builder_add (&builder, "(sisosssss)", app->icon,
 			                       position++, app->dbus_name, app->menu,
 			                       app->icon_theme_path, app->label,
 			                       app->guide,
-			                       (app->icon_desc != NULL) ? app->icon_desc : "");
+			                       (app->icon_desc != NULL) ? app->icon_desc : "",
+			                       app->id);
 		}
 
 		out = g_variant_builder_end(&builder);
 	} else {
 		GError * error = NULL;
-		out = g_variant_parse(g_variant_type_new("a(sisossss)"), "[]", NULL, NULL, &error);
+		out = g_variant_parse(g_variant_type_new("a(sisosssss)"), "[]", NULL, NULL, &error);
 		if (error != NULL) {
-			g_warning("Unable to parse '[]' as a 'a(sisossss)': %s", error->message);
+			g_warning("Unable to parse '[]' as a 'a(sisosssss)': %s", error->message);
 			out = NULL;
 			g_error_free(error);
 		}
